#include <PalmOS.h>
#include "T-Suite.h"
#include "PalmChars.h"

#define TimeSelectorForm						10200
#define TimeSelectorCheckMarkLabel				10201
#define TimeSelectorStartTimePushButton			10204
#define TimeSelectorEndTimePushButton			10205
#define TimeSelectorHourList					10206
#define TimeSelectorMinuteList					10207
#define TimeSelectorMinuteDummyList				10219
#define TimeSelectorStartTimeMinIncRepeating	10220
#define TimeSelectorStartTimeMinDecRepeating	10221
#define TimeSelectorEndTimeMinIncRepeating		10223
#define TimeSelectorEndTimeMinDecRepeating		10224

#define vchrJogUp				0x1700
#define vchrJogDown				0x1701
#define vchrJogPushedUp			0x1703
#define vchrJogPushedDown		0x1704
#define vchrJogRelease			0x1706
#define vchrJogBack				0x1707
#define vchrJogLeft				0x1708
#define vchrJogRight			0x1709


static UInt16 minuteToSelection (UInt16 minute)
{
	UInt16 A = (minute + 4) / 5;
	UInt16 B = (A * 5) - minute;
	UInt16 ret = (51 * B) + A;
	return ret;
}

Err SelTEventDequedHandle (SysNotifyParamType *notifyParamsP)
{
	EventType *event = (EventType*)notifyParamsP->notifyDetailsP;
	EventType newEvent;
	eventsEnum eType = (eventsEnum)Swap16 (event->eType);

//	// check Event to Handle and FormID (TimeSelectorForm)
	switch (eType)
	{
	case nilEvent:
	case keyDownEvent:
	case winEnterEvent:
	case ctlSelectEvent:
	case ctlRepeatEvent:
	case lstEnterEvent:
	case lstSelectEvent:
		UInt16 frmID = FrmGetActiveFormID ();
		if (frmID != TimeSelectorForm)
			return errNone;
		break;
	default:
		return errNone;
	}

//	// get object pointer and Control status
//		// Form object pointer
	FormType *formP = FrmGetActiveForm ();
//		// List object pointer
	ListPtr hoursLst = (ListPtr)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorHourList));
	ListPtr minutesLst = (ListPtr)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorMinuteList));
	ListPtr minutesDmyLst = (ListPtr)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorMinuteDummyList));
//		// Control object pointer
	ControlPtr startTimeCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorStartTimePushButton));
	ControlPtr endTimeCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorEndTimePushButton));
//		// Control object status
	Int16 startTimeCtlValue = CtlGetValue (startTimeCtl);
	Int16 endTimeCtlValue = CtlGetValue (endTimeCtl);

//	// Event Handle
//		// winEnterEvent to initialize Dummy List selection
//		// nilEvent to update Dummy List selection
	if ((eType == winEnterEvent) || (eType == nilEvent))
	{
		LstSetSelection (minutesDmyLst, (UInt8)(LstGetSelection (minutesLst) * 5) / 5);
	}
//		// lstEnterEvent Accept or Not
	if (eType == lstEnterEvent)
	{
		if (Swap16 (event->data.lstEnter.listID) == TimeSelectorMinuteDummyList)
		{
			if (LstGetSelection (minutesLst) == noListSelection)
			{
				SndPlaySystemSound (sndError);
				event->eType = nilEvent;
				return errNone;
			}
		}
	}
//		// lstSelectEvent to Update actual minutesLst
	if (eType == lstSelectEvent)
	{
		if (Swap16 (event->data.lstSelect.listID) == TimeSelectorMinuteDummyList)
		{
			LstSetSelection (minutesLst, LstGetSelection (minutesDmyLst));
			newEvent.eType = lstSelectEvent;
			newEvent.data.lstSelect.listID = TimeSelectorMinuteList;
			EvtAddEventToQueue (&newEvent);
			return errNone;
		}
	}
//		// ctlSelectEvent to check StartTime or EndTimePushButton
//		// if true need update minutesDmyList
	if (eType == ctlSelectEvent)
	{
		switch Swap16 (event->data.ctlSelect.controlID)
		{
		case TimeSelectorStartTimePushButton:
		case TimeSelectorEndTimePushButton:
			newEvent.eType = nilEvent;
			EvtAddEventToQueue (&newEvent);
			return errNone;
		}
	}

//	// check tab Char or 5-Way Left,Right
	if (eType == keyDownEvent)
	{
		newEvent.eType = (eventsEnum)Swap16 (event->eType);
		newEvent.data.keyDown.chr = Swap16 (event->data.keyDown.chr);
		newEvent.data.keyDown.keyCode = Swap16 (event->data.keyDown.keyCode);
		newEvent.data.keyDown.modifiers = Swap16 (event->data.keyDown.modifiers);
		if ((Swap16 (event->data.keyDown.chr) == tabChr) ||
			(Swap16 (event->data.keyDown.chr) == chrLeftArrow) ||
			(Swap16 (event->data.keyDown.chr) == chrRightArrow) ||
			(Swap16 (event->data.keyDown.chr) == vchrJogLeft) ||
			(Swap16 (event->data.keyDown.chr) == vchrJogRight) ||
			NavKeyPressed(&newEvent, Right) || NavKeyPressed(&newEvent, Left))
			{
				event->data.keyDown.chr = Swap16 (tabChr);
			}
		if (LstGetSelection (minutesLst) == noListSelection)
		{
			if ((Swap16 (event->data.keyDown.chr) == tabChr) ||
				(Swap16 (event->data.keyDown.chr) == chrUpArrow) ||
				(Swap16 (event->data.keyDown.chr) == chrDownArrow) ||
				(Swap16 (event->data.keyDown.chr) == chrPageUp) ||
				(Swap16 (event->data.keyDown.chr) == chrPageDown))
			{
				CtlHitControl (startTimeCtl);
				CtlHitControl (startTimeCtl);
			}
		}
	}
//	// check current selection in minutesList
	if (LstGetSelection (minutesLst) == noListSelection)
			return errNone;
//		// keyDownEvent ctlSelectEvent ctlRepeatEvent
	if ((eType == keyDownEvent) ||
		(eType == ctlSelectEvent) ||
		(eType == ctlRepeatEvent))
	{
//		// check which Control is Active End or Not
		Char *CtlLabelP;
		if (endTimeCtlValue)
			CtlLabelP = (char*)CtlGetLabel (endTimeCtl);
		else
			CtlLabelP = (char*)CtlGetLabel (startTimeCtl);
//		// check ControlID to Handle or Not
		if ((eType == ctlSelectEvent) || (eType == ctlRepeatEvent))
		{
			switch Swap16 (event->data.ctlSelect.controlID)
			{
			case TimeSelectorStartTimePushButton:
			case TimeSelectorEndTimePushButton:
				break;
			case TimeSelectorStartTimeMinIncRepeating:
			case TimeSelectorStartTimeMinDecRepeating:
				if (((!startTimeCtlValue) && (!endTimeCtlValue)) || (endTimeCtlValue))
				{
					ControlPtr startTimeMinIncCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorStartTimeMinIncRepeating));
					ControlPtr startTimeMinDecCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorStartTimeMinDecRepeating));
					CtlSetValue (startTimeMinIncCtl, true);
					CtlSetValue (startTimeMinDecCtl, true);
					CtlSetValue (startTimeMinIncCtl, false);
					CtlSetValue (startTimeMinDecCtl, false);
					CtlHitControl (startTimeCtl);
					CtlHitControl (startTimeCtl);
					event->eType = nilEvent;
					return errNone;
				}
				break;
			case TimeSelectorEndTimeMinIncRepeating:
			case TimeSelectorEndTimeMinDecRepeating:
				if (((!startTimeCtlValue) && (!endTimeCtlValue)) || (startTimeCtlValue))
				{
					ControlPtr endTimeMinIncCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorEndTimeMinIncRepeating));
					ControlPtr endTimeMinDecCtl = (ControlType*)FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, TimeSelectorEndTimeMinDecRepeating));
					CtlSetValue (endTimeMinIncCtl, true);
					CtlSetValue (endTimeMinDecCtl, true);
					CtlSetValue (endTimeMinIncCtl, false);
					CtlSetValue (endTimeMinDecCtl, false);
					CtlHitControl (endTimeCtl);
					CtlHitControl (endTimeCtl);
					event->eType = nilEvent;
					return errNone;
				}
				break;
			default:
				return errNone;
			}
		}
//		// get minutes value
		Char *minutesP = StrChr (CtlLabelP, ':') + 1;
		if (minutesP == (Char*)1)
			minutesP = StrChr (CtlLabelP, '.') + 1;
		if (minutesP == (Char*)1)
			minutesP = StrChr (CtlLabelP, ',') + 1;

		Int32 min = StrAToI (minutesP);

		if (eType == keyDownEvent)
		{
			switch Swap16 (event->data.keyDown.chr)
			{
			case tabChr:
				if ((!startTimeCtlValue) && (!endTimeCtlValue))
					EvtEnqueueKey (nextFieldChr, 0, 0);
				event->data.keyDown.chr = Swap16 (nextFieldChr);
				return errNone;
			case backspaceChr:
				LstSetSelection (hoursLst, noListSelection);
				LstSetSelection (minutesLst, noListSelection);
				LstSetSelection (minutesDmyLst, noListSelection);
				return errNone;
			case chrUpArrow:
			case chrPageUp:
				goto Inc;
			case chrDownArrow:
			case chrPageDown:
				goto Dec;
			case vchrJogRelease:
				event->data.keyDown.chr = Swap16 (linefeedChr);
				return errNone;
			}
		}

		if (eType == ctlRepeatEvent)
		{
			switch Swap16 (event->data.ctlRepeat.controlID)
			{
			case TimeSelectorStartTimeMinIncRepeating:
				if (!startTimeCtlValue)
					return errNone;
				goto Inc;
			case TimeSelectorEndTimeMinIncRepeating:
				if (!endTimeCtlValue)
					return errNone;
				goto Inc;
			case TimeSelectorStartTimeMinDecRepeating:
				if (!startTimeCtlValue)
					return errNone;
				goto Dec;
			case TimeSelectorEndTimeMinDecRepeating:
				if (!endTimeCtlValue)
					return errNone;
				goto Dec;
			default:
				return errNone;
			}
		}
		return errNone;

Inc:
		min++;
		if (min == 60)
		{
			if (LstGetSelection (hoursLst) != 23)
			{
				LstSetSelection (hoursLst, LstGetSelection (hoursLst) + 1);
				newEvent.eType = lstSelectEvent;
				newEvent.data.lstSelect.listID = TimeSelectorHourList;
				EvtAddEventToQueue (&newEvent);
				min = 0;
			}
			else
			{
				return errNone;
			}
		}
		goto IncDecHandled;

Dec:
		min--; 
		if (min == -1)
		{
			if (LstGetSelection (hoursLst) != 0)
			{
				LstSetSelection (hoursLst, LstGetSelection (hoursLst) - 1);
				newEvent.eType = lstSelectEvent;
				newEvent.data.lstSelect.listID = TimeSelectorHourList;
				EvtAddEventToQueue (&newEvent);
				min = 59;
			}
			else
			{
				return errNone;
			}
		}

IncDecHandled:
		UInt16 sel = minuteToSelection (min);

		LstSetSelection (minutesLst, sel);
		LstSetSelection (minutesDmyLst, min / 5);

		newEvent.eType = lstSelectEvent;
		newEvent.data.lstSelect.listID = TimeSelectorMinuteList;
		EvtAddEventToQueue (&newEvent);
	}

	notifyParamsP->handled = true;

	return errNone;
}